import argparse
import atexit
import glob
import logging
import os
import sys
import traceback

# on Windows, any logging that happens before we set up our log file ends up going to a text file
# and the app spits out a message box after the run telling the user to check it. So this silences that.
if sys.platform.startswith("win"):
    logging.basicConfig(filename=os.devnull, filemode='w', level=logging.DEBUG)

from constants import *
from settings import *

parser = argparse.ArgumentParser(description='Install Toolfarm packages.')
parser.add_argument('--list', help="List installed packages", action="store_true")
parser.add_argument('--show_files', help="Show files installed for package [package]")
parser.add_argument('--uninstall', help="Uninstall package [package]", action="store_true")
parser.add_argument('--version', action='version', version='%(prog)s ' + VERSION)
parser.add_argument('--silent', action="store_true", help="Install the package without showing the GUI.")

parser.add_argument('filename', help="Filename of package to install", default="", nargs="?")

args = None

# a -psn argument means we're gui, so run the gui app
for pos, arg in enumerate(sys.argv):
    if arg.startswith('-psn'):
        sys.argv.remove(arg)
        break

try:
    args = parser.parse_args()
except:
    parser.print_help()
    sys.exit(1)

package = ""
if args.filename and args.filename.endswith(".tfi"):
    package = args.filename
else:
    files = glob.glob(os.path.abspath("*.tfi"))
    if len(files) > 0:
        package = files[0]

log_name = os.path.basename(sys.executable).replace(".exe", "") + ".log"
error_msg = None
if package:
    try:
        import installer.package
        pkg = installer.package.TFPackage(package)
        if "publisher" in pkg.package_info:
            publisher = pkg.package_info["publisher"]
            set_publisher(publisher)
        if "package_name" in pkg.package_info and "package_version" in pkg.package_info:
            log_name = "%s %s Installer.log" % (pkg.package_info["package_name"], pkg.package_info["package_version"])
    except Exception, e:
        error_msg = traceback.format_exc(e)

import logs
logs.initialize_log(log_name)
logging.info("Log file initialized")
if error_msg is not None:
    logging.error(error_msg)

import errors

import controllers.app
import installer.install
import installer.install_manager
import installer.install_process
import ui

app_dir = os.path.dirname(os.path.abspath(sys.executable))
os.environ['PATH'] += os.pathsep + app_dir

atexit.register(installer.install_process.shutdown)

logging.info("Are we starting?")

try:
    logging.info("filename = %r" % args.filename)
    manager = installer.install_manager.InstallManager()
    
    if args.list:
        installs = manager.get_installations()
        for install in installs:
            print("%s %s" % (install[0], install[1]))
    elif args.show_files is not None:
        id = manager.get_installation_id(args.show_files)
        files = manager.get_files(args.show_files)
        print(files)
    elif args.silent and args.uninstall is not None:
        uninstaller = installer.install.TFUninstallController(args.uninstall)
        success = uninstaller.do_uninstall()
        if success:
            print("Package uninstalled!")
        else:
            print("Package uninstall failed.")
    else:
        
        if args.silent:
            installer = installer.install.TFInstallController(package)
            installer.do_install()
        else:
            controller = controllers.app.AppController(package)
            controller.start_app(args.filename)
        
except Exception, e:
    import traceback
    logging.error(traceback.format_exc(e))
